/*
 * Copyright (C) 1999-2022. Christian Heller.
 *
 * This file is part of the Cybernetics Oriented Interpreter (CYBOI).
 *
 * CYBOI is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * CYBOI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CYBOI. If not, see <http://www.gnu.org/licenses/>.
 *
 * Cybernetics Oriented Programming (CYBOP) <http://www.cybop.org/>
 * CYBOP Developers <cybop-developers@nongnu.org>
 *
 * @version CYBOP 0.23.0 2022-09-04
 * @author Christian Heller <christian.heller@cybop.org>
 */

#ifndef WHITESPACE_NORMALISER_SOURCE
#define WHITESPACE_NORMALISER_SOURCE

#include "../../../constant/format/cyboi/logic_cyboi_format.c"
#include "../../../constant/model/character_code/unicode/unicode_character_code_model.c"
#include "../../../constant/model/cyboi/log/level_log_cyboi_model.c"
#include "../../../constant/model/cyboi/state/boolean_state_cyboi_model.c"
#include "../../../constant/model/cyboi/state/integer_state_cyboi_model.c"
#include "../../../constant/model/cyboi/state/pointer_state_cyboi_model.c"
#include "../../../constant/model/cyboi/state/state_cyboi_model.c"
#include "../../../constant/name/cyboi/state/primitive_state_cyboi_name.c"
#include "../../../constant/type/cyboi/state_cyboi_type.c"
#include "../../../executor/comparator/integer/less_or_equal_integer_comparator.c"
#include "../../../executor/copier/integer_copier.c"
#include "../../../executor/modifier/normaliser/non_whitespace_selector_normaliser.c"
#include "../../../executor/modifier/array_modifier.c"
#include "../../../logger/logger.c"

/**
 * Parses through the whitespace characters of the given source array.
 *
 * If a non-whitespace character is found, then the parsing stops
 * and only ONE space character is written into the destination,
 * no matter how many whitespaces had been found in the source before.
 *
 * @param p0 the destination array (pointer reference)
 * @param p1 the destination array count
 * @param p2 the destination array size
 * @param p3 the source data position (pointer reference)
 * @param p4 the source count remaining
 */
void normalise_whitespace(void* p0, void* p1, void* p2, void* p3, void* p4) {

    log_message_terminated((void*) DEBUG_LEVEL_LOG_CYBOI_MODEL, (void*) L"Normalise whitespace.");
/*??
    fwprintf(stdout, L"Debug: Normalise whitespace. count remaining p4: %i\n", p4);
    fwprintf(stdout, L"Debug: Normalise whitespace. count remaining *p4: %i\n", *((int*) p4));
    fwprintf(stdout, L"Debug: Normalise whitespace. data position *p3: %i\n", *((void**) p3));
    fwprintf(stdout, L"Debug: Normalise whitespace. data position *p3 ls: %ls\n", (wchar_t*) *((void**) p3));
    fwprintf(stdout, L"Debug: Normalise whitespace. data position *p3 lc: %lc\n", *((wchar_t*) *((void**) p3)));
    fwprintf(stdout, L"Debug: Normalise whitespace. data position *p3 lc as int: %i\n", *((wchar_t*) *((void**) p3)));
*/

    // The break flag.
    int b = *FALSE_BOOLEAN_STATE_CYBOI_MODEL;

    if (p4 == *NULL_POINTER_STATE_CYBOI_MODEL) {

        //
        // CAUTION! If the loop count handed over as parametre is NULL,
        // then the break flag will NEVER be set to true, because the loop
        // variable comparison does (correctly) not consider null values.
        // Therefore, in this case, the break flag is set to true already here.
        //
        // Initialising the break flag with true will NOT work either, since it:
        // a) will be left untouched if a comparison operand is null;
        // b) would have to be reset to true in each loop cycle.
        //
        copy_integer((void*) &b, (void*) TRUE_BOOLEAN_STATE_CYBOI_MODEL);
    }

    while (*TRUE_BOOLEAN_STATE_CYBOI_MODEL) {

        compare_integer_less_or_equal((void*) &b, p4, (void*) NUMBER_0_INTEGER_STATE_CYBOI_MODEL);

        if (b != *FALSE_BOOLEAN_STATE_CYBOI_MODEL) {

            //
            // Either a NON-whitespace character delimiting this
            // sequence of whitespace characters has been found
            // OR the source count remaining is ZERO.
            // In BOTH cases, a space character is to be added.
            //
            // This function "normalise_whitespace" was entered only
            // because a space character was detected, so that
            // at least one space character is present in the source
            // and in BOTH cases mentioned above, also if the
            // source count remaining is zero, a space is to be added.
            //

            // Append ONE space character to destination.
            modify_array(p0, (void*) SPACE_UNICODE_CHARACTER_CODE_MODEL, (void*) WIDE_CHARACTER_TEXT_STATE_CYBOI_TYPE, (void*) FALSE_BOOLEAN_STATE_CYBOI_MODEL, (void*) PRIMITIVE_STATE_CYBOI_MODEL_COUNT, *NULL_POINTER_STATE_CYBOI_MODEL, (void*) VALUE_PRIMITIVE_STATE_CYBOI_NAME, p1, p2, (void*) TRUE_BOOLEAN_STATE_CYBOI_MODEL, (void*) APPEND_MODIFY_LOGIC_CYBOI_FORMAT);

            break;
        }

        // Search for non-whitespace character.
        normalise_select_non_whitespace((void*) &b, p3, p4);
    }
}

/* WHITESPACE_NORMALISER_SOURCE */
#endif
